@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as
@@ -30,9 +30,9 @@
  * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
  */
 
-#include <libxfs.h>
+#include <xfs/libxfs.h>
+#include <xfs/jdm.h>
 #include <malloc.h>
-#include <jdm.h>
 
 #include <sys/stat.h>
 #include <errno.h>
@@ -56,6 +56,7 @@
 #include "inomap.h"
 #include "arch_xlate.h"
 #include "exit.h"
+#include <attr/attributes.h>
 
 /* structure definitions used locally ****************************************/
 
@@ -75,15 +76,16 @@ extern size_t pgsz;
 extern hsm_fs_ctxt_t *hsm_fs_ctxtp;
 #endif /* DMEXTATTR */
 extern u_int64_t maxdumpfilesize;
+extern bool_t allowexcludefiles_pr;
 
 /* forward declarations of locally defined static functions ******************/
 
 /* inomap construction callbacks
  */
 static void cb_context( bool_t last,
-			time_t,
+			time32_t,
 			bool_t,
-			time_t,
+			time32_t,
 			size_t,
 			drange_t *,
 			size_t,
@@ -171,6 +173,8 @@ static void mln_free( void );
 static ix_t *inomap_statphasep;
 static ix_t *inomap_statpassp;
 static size64_t *inomap_statdonep;
+static u_int64_t inomap_exclude_filesize = 0;
+static u_int64_t inomap_exclude_skipattr = 0;
 
 /* definition of locally defined global functions ****************************/
 
@@ -184,9 +188,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 	      intgen_t fsfd,
 	      xfs_bstat_t *rootstatp,
 	      bool_t last,
-	      time_t lasttime,
+	      time32_t lasttime,
 	      bool_t resume,
-	      time_t resumetime,
+	      time32_t resumetime,
 	      size_t resumerangecnt,
 	      drange_t *resumerangep,
 	      char *subtreebuf[],
@@ -230,9 +234,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 	 * this will be used during the tree pruning phase.
 	 */
 	if ( subtreecnt ) {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 1: "
-		      "parsing subtree selections\n" );
+		      "parsing subtree selections\n") );
 		if ( ! subtreelist_parse( fshandlep,
 					  fsfd,
 					  rootstatp,
@@ -243,9 +247,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 			return BOOL_FALSE;
 		}
 	} else {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 1: "
-		      "skipping (no subtrees specified)\n" );
+		      "skipping (no subtrees specified)\n") );
 	}
 	
 	if ( preemptchk( PREEMPT_FULL )) {
@@ -278,9 +282,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 	 * callback. set a flag if any ino not put in a dump state. This will
 	 * be used to decide if any pruning can be done.
 	 */
-	mlog( MLOG_VERBOSE | MLOG_INOMAP,
+	mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 	      "ino map phase 2: "
-	      "constructing initial dump list\n" );
+	      "constructing initial dump list\n") );
 	*inomap_statdonep = 0;
 	*inomap_statphasep = 2;
 	pruneneeded = BOOL_FALSE;
@@ -309,6 +313,17 @@ inomap_build( jdm_fshandle_t *fshandlep,
 		return BOOL_FALSE;
 	}
 
+	if ( inomap_exclude_filesize > 0 ) {
+		mlog( MLOG_NOTE | MLOG_VERBOSE, _(
+		      "pruned %llu files: maximum size exceeded\n"),
+		      inomap_exclude_filesize );
+	}
+	if ( inomap_exclude_skipattr > 0 ) {
+		mlog( MLOG_NOTE | MLOG_VERBOSE, _(
+		      "pruned %llu files: skip attribute set\n"),
+		      inomap_exclude_skipattr );
+	}
+
 	/* prune subtrees not called for in the subtree list, and
 	 * directories unchanged since the last dump and containing
 	 * no children needing dumping. Each time the latter pruning
@@ -322,9 +337,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 		mln_init( );
 		gdrecursearray_init( );
 
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 3: "
-		      "pruning unneeded subtrees\n" );
+		      "pruning unneeded subtrees\n") );
 		*inomap_statdonep = 0;
 		*inomap_statpassp = 0;
 		*inomap_statphasep = 3;
@@ -375,9 +390,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 		cb_postmortem( );
 
 	} else {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 3: "
-		      "skipping (no pruning necessary)\n" );
+		      "skipping (no pruning necessary)\n") );
 	}
 
 	/* free the subtree list memory
@@ -400,9 +415,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 	if ( pruneneeded || subtreecnt > 0 ) {
 		cb_accuminit_sz( );
 
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 4: "
-		      "estimating dump size\n" );
+		      "estimating dump size\n") );
 		*inomap_statdonep = 0;
 		*inomap_statphasep = 4;
 		stat = 0;
@@ -433,9 +448,9 @@ inomap_build( jdm_fshandle_t *fshandlep,
 
 		inomap_state_postaccum( inomap_state_contextp );
 	} else {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 4: "
-		      "skipping (size estimated in phase 2)\n" );
+		      "skipping (size estimated in phase 2)\n") );
 	}
 
 	/* initialize the callback context for startpoint calculation
@@ -445,13 +460,13 @@ inomap_build( jdm_fshandle_t *fshandlep,
 	/* identify dump stream startpoints
 	 */
 	if ( startptcnt > 1 ) {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 5: "
-		      "identifying stream starting points\n" );
+		      "identifying stream starting points\n") );
 	} else {
-		mlog( MLOG_VERBOSE | MLOG_INOMAP,
+		mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
 		      "ino map phase 5: "
-		      "skipping (only one dump stream)\n" );
+		      "skipping (only one dump stream)\n") );
 	}
 	stat = 0;
 	*inomap_statdonep = 0;
@@ -491,17 +506,19 @@ inomap_build( jdm_fshandle_t *fshandlep,
 				ep = &startptp[ startptix + 1 ];
 			}
 			ASSERT( ! p->sp_flags );
-			mlog( MLOG_VERBOSE | MLOG_INOMAP,
-			      "stream %u: ino %llu offset %lld to ",
-			      startptix,
-			      p->sp_ino,
-			      p->sp_offset );
 			if ( ! ep ) {
-				mlog( MLOG_VERBOSE | MLOG_BARE | MLOG_INOMAP,
-				      "end\n" );
+				mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
+				   "stream %u: ino %llu offset %lld to end\n"),
+				      startptix,
+				      p->sp_ino,
+				      p->sp_offset );
 			} else {
-				mlog( MLOG_VERBOSE |  MLOG_BARE | MLOG_INOMAP,
-				      "ino %llu offset %lld\n",
+				mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
+				    "stream %u: ino %llu offset %lld to "
+				    "ino %llu offset %lld\n"),
+				      startptix,
+				      p->sp_ino,
+				      p->sp_offset,
 				      ep->sp_ino,
 				      ep->sp_offset );
 			}
@@ -510,8 +527,8 @@ inomap_build( jdm_fshandle_t *fshandlep,
 
 	free( ( void * )bstatbufp );
 	free( ( void * )getdentbufp );
-	mlog( MLOG_VERBOSE | MLOG_INOMAP,
-	      "ino map construction complete\n" );
+	mlog( MLOG_VERBOSE | MLOG_INOMAP, _(
+	      "ino map construction complete\n") );
 	return BOOL_TRUE;
 }
 
@@ -539,9 +556,9 @@ inomap_skip( xfs_ino_t ino )
  * of iterators. below are the callbacks given to these iterators.
  */
 static bool_t cb_last;		/* set by cb_context() */
-static time_t cb_lasttime;	/* set by cb_context() */
+static time32_t cb_lasttime;	/* set by cb_context() */
 static bool_t cb_resume;	/* set by cb_context() */
-static time_t cb_resumetime;	/* set by cb_context() */
+static time32_t cb_resumetime;	/* set by cb_context() */
 static size_t cb_resumerangecnt;/* set by cb_context() */
 static drange_t *cb_resumerangep;/* set by cb_context() */
 static ix_t cb_subtreecnt;	/* set by cb_context() */
@@ -564,9 +581,9 @@ static size_t cb_maxrecursionlevel;
  */
 static void
 cb_context( bool_t last,
-	    time_t lasttime,
+	    time32_t lasttime,
 	    bool_t resume,
-	    time_t resumetime,
+	    time32_t resumetime,
 	    size_t resumerangecnt,
 	    drange_t *resumerangep,
 	    ix_t subtreecnt,
@@ -597,8 +614,8 @@ cb_context( bool_t last,
 static void
 cb_postmortem( void )
 {
-	mlog( MLOG_DEBUG | MLOG_NOTE | MLOG_INOMAP,
-	      "maximum subtree pruning recursion depth: %u\n",
+	mlog( MLOG_DEBUG | MLOG_NOTE | MLOG_INOMAP, _(
+	      "maximum subtree pruning recursion depth: %u\n"),
 	      cb_maxrecursionlevel );
 }
 
@@ -614,20 +631,29 @@ cb_add( void *arg1,
 	intgen_t fsfd,
 	xfs_bstat_t *statp )
 {
-	register time_t mtime = statp->bs_mtime.tv_sec;
-	register time_t ctime = statp->bs_ctime.tv_sec;
-	register time_t ltime = max( mtime, ctime );
+	register time32_t mtime = statp->bs_mtime.tv_sec;
+	register time32_t ctime = statp->bs_ctime.tv_sec;
+	register time32_t ltime = max( mtime, ctime );
 	register mode_t mode = statp->bs_mode & S_IFMT;
 	xfs_off_t estimated_size = 0;
 	xfs_ino_t ino = statp->bs_ino;
 	bool_t changed;
 	bool_t resumed;
 
+#ifdef DEBUG_INOMAP
+	mlog( MLOG_DEBUG | MLOG_INOMAP, "cb_add: ino %llu,"
+                                "mtime = %d, ctime = %d\n", 
+                                 ino, mtime, ctime);
+#endif
+
 	( *inomap_statdonep )++;
 
 	/* skip if no links
 	 */
 	if ( statp->bs_nlink == 0 ) {
+#ifdef DEBUG_INOMAP
+		mlog( MLOG_DEBUG | MLOG_INOMAP, "cb_add: skip as no links\n");
+#endif
 		return 0;
 	}
 
@@ -642,14 +668,34 @@ cb_add( void *arg1,
 	 */
 	if ( cb_resume && ! cb_inoinresumerange( ino )) {
 		if ( ltime >= cb_resumetime ) {
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+                              "cb_add/resume: changed ltime %d, resume %d\n",
+                                ltime, cb_resumetime);
+#endif
 			changed = BOOL_TRUE;
 		} else {
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+				"cb_add/resume: NOT changed ltime %d, resume %d\n", 
+				ltime, cb_resumetime);
+#endif
 			changed = BOOL_FALSE;
 		}
 	} else if ( cb_last ) {
 		if ( ltime >= cb_lasttime ) {
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+			    "cb_add/last: changed ltime %d, last %d\n", 
+			    ltime, cb_lasttime);
+#endif
 			changed = BOOL_TRUE;
 		} else {
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+			    "cb_add/last: NOT changed ltime %d, last %d\n", 
+			    ltime, cb_lasttime);
+#endif
 			changed = BOOL_FALSE;
 		}
 	} else {
@@ -666,6 +712,9 @@ cb_add( void *arg1,
 	}
 
 	if ( changed ) {
+#ifdef DEBUG_INOMAP
+		mlog( MLOG_DEBUG | MLOG_INOMAP, "cb_add: changed %llu\n", ino);
+#endif
 		if ( mode == S_IFDIR ) {
 			map_add( ino, MAP_DIR_CHANGE );
 			cb_dircnt++;
@@ -675,17 +724,53 @@ cb_add( void *arg1,
 			/* skip if size is greater than prune size
 			 */
 			if ( maxdumpfilesize > 0 &&
-			    estimated_size > maxdumpfilesize ) {
-				mlog( MLOG_DEBUG | MLOG_INOMAP,
-				      "ino %llu pruned, estimated size %llu > maxdumpfilesuze %llu\n",
+			     estimated_size > maxdumpfilesize ) {
+				mlog( MLOG_DEBUG | MLOG_EXCLFILES,
+				      "pruned ino %llu, owner %u, estimated size %llu: maximum size exceeded\n",
 				      statp->bs_ino,
-				      estimated_size,
-				      maxdumpfilesize );
+				      statp->bs_uid,
+				      estimated_size );
 				map_add( ino, MAP_NDR_NOCHNG );
+				inomap_exclude_filesize++;
 				return 0;
 			}
 
+#ifdef EXTATTR
+			if (allowexcludefiles_pr && (statp->bs_xflags & XFS_XFLAG_HASATTR)) {
+				int rval;
+				attr_multiop_t attrop;
+				static char *skip_attr_name = "SGI_XFSDUMP_SKIP_FILE";
+
+				attrop.am_attrname  = skip_attr_name;
+				attrop.am_attrvalue = NULL;
+				attrop.am_length    = 0;
+				attrop.am_error     = 0;
+				attrop.am_flags     = 0;
+				attrop.am_opcode    = ATTR_OP_GET;
+                                
+				rval = jdm_attr_multi( fshandlep,
+						       statp,
+						       (char *)&attrop,
+						       1,
+						       0 );
+				if ( !rval && (!attrop.am_error || attrop.am_error == E2BIG || attrop.am_error == ERANGE) ) {
+					mlog( MLOG_DEBUG | MLOG_EXCLFILES,
+					      "pruned ino %llu, owner %u, estimated size %llu: skip attribute set\n",
+					      statp->bs_ino,
+					      statp->bs_uid,
+					      estimated_size );
+					map_add( ino, MAP_NDR_NOCHNG );
+					inomap_exclude_skipattr++;
+					return 0;
+				}
+                        }
+#endif /* EXTATTR */
+
 			map_add( ino, MAP_NDR_CHANGE );
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+			    "cb_add: map_add ino %llu ndr_chng\n", ino);
+#endif
 			cb_nondircnt++;
 			cb_datasz += estimated_size;
 		}
@@ -699,6 +784,10 @@ cb_add( void *arg1,
 			map_add( ino, MAP_DIR_SUPPRT );
 			cb_dircnt++;
 		} else {
+#ifdef DEBUG_INOMAP
+			mlog( MLOG_DEBUG | MLOG_INOMAP, 
+			    "cb_add: map_add ino %llu ndr_nochng\n", ino);
+#endif
 			map_add( ino, MAP_NDR_NOCHNG );
 		}
 	}
@@ -814,8 +903,16 @@ supprt_prune( void *arg1,	/* ancestors marked as changed? */
 		bool_t changed_below = BOOL_FALSE;
 
 		state = map_get( statp->bs_ino );
-		ASSERT( state != MAP_NDR_CHANGE );
-		ASSERT( state != MAP_NDR_NOCHNG );
+		if ( state != MAP_DIR_CHANGE &&
+                     state != MAP_DIR_NOCHNG &&
+		     state != MAP_DIR_SUPPRT) {
+			/* 
+			 * if file is now a dir then it has
+			 * certainly changed.
+			 */
+			state = MAP_DIR_CHANGE;
+			map_set( statp->bs_ino, state );
+		}
 
 		( void )diriter( fshandlep,
 				 fsfd,
@@ -851,10 +948,15 @@ supprt_prune( void *arg1,	/* ancestors marked as changed? */
 	}
 
 	state = map_get( statp->bs_ino );
-	ASSERT( state != MAP_DIR_CHANGE );
-	ASSERT( state != MAP_DIR_NOCHNG );
-	ASSERT( state != MAP_DIR_SUPPRT );
-
+	if ( state != MAP_NDR_CHANGE &&
+	     state != MAP_NDR_NOCHNG ) {
+		/* 
+		 * if dir is now a file then it has
+		 * certainly changed.
+		 */
+		state = MAP_NDR_CHANGE;
+		map_set( statp->bs_ino, state );
+	}
 	if ( state == MAP_NDR_CHANGE ) {
 		/* Directory entries back up the hierarchy must get */
 		/* dumped - as either MAP_DIR_SUPPRT/MAP_DIR_CHANGE */
@@ -944,9 +1046,9 @@ cb_del( void *arg1,
 	register size_t recursion_level = ( size_t )arg1;
 
 	if ( recursion_level >= GDRECURSEDEPTHMAX ) {
-		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP,
+		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP, _(
 		      "subtree pruning recursion depth exceeds max (%d): "
-		      "some unselected subtrees may be included in dump\n",
+		      "some unselected subtrees may be included in dump\n"),
 		      GDRECURSEDEPTHMAX );
 		return 0;
 	}
@@ -1394,9 +1496,10 @@ map_add( xfs_ino_t ino, intgen_t state )
 	}
 
 	if (ino <= last_ino_added) {
-		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_INOMAP, 
-		  "map_add(%llu, %d): ino(%llu) <= last_ino(%llu)\n",
+		mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_INOMAP, _(
+		  "map_add(%llu, %d): ino(%llu) <= last_ino(%llu)\n"),
 		  ino, state, ino, last_ino_added);
+		mlog_exit(EXIT_ERROR, RV_NONE);
 		exit(EXIT_ERROR);
 	}
 
@@ -1896,8 +1999,8 @@ subtreelist_parse( jdm_fshandle_t *fshandlep,
 		if ( cbrval != 1 ) {
 			mlog( MLOG_NORMAL | MLOG_ERROR | MLOG_INOMAP,
 			      "%s: %s\n",
-			      cbrval == 0 ? "subtree not present"
-					  : "invalid subtree specified",
+			      cbrval == 0 ? _("subtree not present")
+					  : _("invalid subtree specified"),
 			      currentpath );
 			return BOOL_FALSE;
 		}
@@ -2076,8 +2179,8 @@ quantity2offset( jdm_fshandle_t *fshandlep, xfs_bstat_t *statp, off64_t qty )
 	bmap[ 0 ].bmv_entries = -1;
 	fd = jdm_open( fshandlep, statp, O_RDONLY );
 	if ( fd < 0 ) {
-		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP,
-		      "could not open ino %llu to read extent map: %s\n",
+		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP, _(
+		      "could not open ino %llu to read extent map: %s\n"),
 		      statp->bs_ino,
 		      strerror( errno ));
 		return 0;
@@ -2089,8 +2192,8 @@ quantity2offset( jdm_fshandle_t *fshandlep, xfs_bstat_t *statp, off64_t qty )
 
 		rval = ioctl( fd, XFS_IOC_GETBMAPX, bmap );
 		if ( rval ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP,
-			      "could not read extent map for ino %llu: %s\n",
+			mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP, _(
+			      "could not read extent map for ino %llu: %s\n"),
 			      statp->bs_ino,
 			      strerror( errno ));
 			( void )close( fd );
@@ -2154,8 +2257,8 @@ estimate_dump_space( xfs_bstat_t *statp )
 	*/
 		return 0;
 	default:
-		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP,
-		      "unknown inode type: ino=%llu, mode=0x%04x 0%06o\n",
+		mlog( MLOG_NORMAL | MLOG_WARNING | MLOG_INOMAP, _(
+		      "unknown inode type: ino=%llu, mode=0x%04x 0%06o\n"),
 		      statp->bs_ino,
 		      statp->bs_mode,
 		      statp->bs_mode );
