@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2001 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as
@@ -30,8 +30,8 @@
  * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
  */
 
-#include <libxfs.h>
-#include <jdm.h>
+#include <xfs/libxfs.h>
+#include <xfs/jdm.h>
 
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -110,9 +110,9 @@ struct dirattr {
 	mode_t d_mode;
 	uid_t d_uid;
 	gid_t d_gid;
-	time_t d_atime;
-	time_t d_mtime;
-	time_t d_ctime;
+	time32_t d_atime;
+	time32_t d_mtime;
+	time32_t d_ctime;
 	u_int32_t d_xflags;
 	u_int32_t d_extsize;
 	u_int32_t d_dmevmask;
@@ -238,9 +238,9 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 		 */
 		dtp->dt_fd = open( dtp->dt_pathname, O_RDWR );
 		if ( dtp->dt_fd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
 			      "could not find directory attributes file %s: "
-			      "%s\n",
+			      "%s\n"),
 			      dtp->dt_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
@@ -254,16 +254,16 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 				   O_RDWR | O_CREAT | O_EXCL,
 				   S_IRUSR | S_IWUSR );
 		if ( dtp->dt_fd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
 			      "could not create directory attributes file %s: "
-			      "%s\n",
+			      "%s\n"),
 			      dtp->dt_pathname,
 			      strerror( errno ));
 			return BOOL_FALSE;
 		}
 
-		/* reserve space for the backing store. try to use F_RESVSP64.
-		 * if doesn't work, try F_RESVSP64. the former is faster, since
+		/* reserve space for the backing store. try to use RESVSP64.
+		 * if doesn't work, try ALLOCSP64. the former is faster, as
 		 * it does not zero the space.
 		 */
 		{
@@ -272,10 +272,6 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 		intgen_t loglevel;
 		size_t trycnt;
 
-#ifndef F_RESVSP64
-#define F_RESVSP64 0
-#endif /* F_RESVSP64 */
-
 		for ( trycnt = 0,
 		      successpr = BOOL_FALSE,
 		      ioctlcmd = XFS_IOC_RESVSP64,
@@ -302,19 +298,21 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 			flock64.l_len = initsz;
 			rval = ioctl( dtp->dt_fd, ioctlcmd, &flock64 );
 			if ( rval ) {
-				mlog( loglevel | MLOG_NOTE,
-				      "attempt to reserve %lld bytes for %s "
-				      "using %s "
-				      "failed: %s (%d)\n",
-				      initsz,
-				      dtp->dt_pathname,
-				      ioctlcmd == F_RESVSP64
-				      ?
-				      "F_RESVSP64"
-				      :
-				      "F_ALLOCSP64",
-				      strerror( errno ),
-				      errno );
+				if ( errno != ENOTTY ) {
+					mlog( loglevel | MLOG_NOTE, _(
+					      "attempt to reserve %lld bytes for %s "
+					      "using %s "
+					      "failed: %s (%d)\n"),
+					      initsz,
+					      dtp->dt_pathname,
+					      ioctlcmd == XFS_IOC_RESVSP64
+					      ?
+					      "XFS_IOC_RESVSP64"
+					      :
+					      "XFS_IOC_ALLOCSP64",
+					      strerror( errno ),
+					      errno );
+				}
 			} else {
 				successpr = BOOL_TRUE;
 			}
@@ -330,8 +328,8 @@ dirattr_init( char *hkdir, bool_t resume, u_int64_t dircnt )
 				        ( off_t )0 );
 	ASSERT( dpp );
 	if ( dpp == ( dirattr_pers_t * )-1 ) {
-		mlog( MLOG_NORMAL | MLOG_ERROR,
-		      "unable to map %s: %s\n",
+		mlog( MLOG_NORMAL | MLOG_ERROR, _(
+		      "unable to map %s: %s\n"),
 		      dtp->dt_pathname,
 		      strerror( errno ));
 		return BOOL_FALSE;
@@ -421,8 +419,8 @@ dirattr_add( filehdr_t *fhdrp )
 		off64_t newoff;
 		newoff = lseek64( dtp->dt_fd, dpp->dp_appendoff, SEEK_SET );
 		if ( newoff == ( off64_t )-1 ) {
-			mlog( MLOG_NORMAL | MLOG_ERROR,
-			      "lseek of dirattr failed: %s\n",
+			mlog( MLOG_NORMAL | MLOG_ERROR, _(
+			      "lseek of dirattr failed: %s\n"),
 			      strerror( errno ));
 			return DAH_NULL;
 		}
@@ -441,9 +439,9 @@ dirattr_add( filehdr_t *fhdrp )
 	dirattr.d_mode = ( mode_t )fhdrp->fh_stat.bs_mode;
 	dirattr.d_uid = ( uid_t )fhdrp->fh_stat.bs_uid;
 	dirattr.d_gid = ( gid_t )fhdrp->fh_stat.bs_gid;
-	dirattr.d_atime = ( time_t )fhdrp->fh_stat.bs_atime.tv_sec;
-	dirattr.d_mtime = ( time_t )fhdrp->fh_stat.bs_mtime.tv_sec;
-	dirattr.d_ctime = ( time_t )fhdrp->fh_stat.bs_ctime.tv_sec;
+	dirattr.d_atime = ( time32_t )fhdrp->fh_stat.bs_atime.tv_sec;
+	dirattr.d_mtime = ( time32_t )fhdrp->fh_stat.bs_mtime.tv_sec;
+	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
@@ -461,8 +459,8 @@ dirattr_add( filehdr_t *fhdrp )
 	 */
 	nwritten = write( dtp->dt_fd, ( void * )&dirattr, sizeof( dirattr ));
 	if ( ( size_t )nwritten != sizeof( dirattr )) {
-		mlog( MLOG_NORMAL | MLOG_ERROR,
-		      "write of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL | MLOG_ERROR, _(
+		      "write of dirattr failed: %s\n"),
 		      strerror( errno ));
 		return DAH_NULL;
 	}
@@ -506,10 +504,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 					  O_RDWR | O_CREAT,
 					  S_IRUSR | S_IWUSR );
 		if ( dtp->dt_extattrfd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not open/create directory "
 			      "extended attributes file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -525,10 +523,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	while ( off != DIRATTR_EXTATTROFFNULL ) {
 		seekoff = lseek64( dtp->dt_extattrfd, off, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -543,10 +541,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 			      ( void * )&off,
 			      sizeof( off ));
 		if ( nread != ( intgen_t )sizeof( off )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -559,10 +557,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	 */
 	off = lseek64( dtp->dt_extattrfd, 0, SEEK_END );
 	if ( off < 0 ) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not seek to end of extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -574,10 +572,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 			  ( void * )&nulloff,
 			  sizeof( nulloff ));
 	if ( nwritten != ( intgen_t )sizeof( nulloff )) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not write extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -586,10 +584,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	}
 	nwritten = write( dtp->dt_extattrfd, ( void * )ahdrp, ahdrp->ah_sz );
 	if ( nwritten != ( intgen_t )( ahdrp->ah_sz )) {
-		mlog( MLOG_NORMAL | MLOG_WARNING,
+		mlog( MLOG_NORMAL | MLOG_WARNING, _(
 		      "could not write at end of extended attributes "
 		      "file %s: "
-		      "%s (%d)\n",
+		      "%s (%d)\n"),
 		      dtp->dt_extattrpathname,
 		      strerror( errno ),
 		      errno );
@@ -606,10 +604,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 	} else {
 		seekoff = lseek64( dtp->dt_extattrfd, oldoff, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -621,10 +619,10 @@ dirattr_addextattr( dah_t dah, extattrhdr_t *ahdrp )
 				  ( void * )&off,
 				  sizeof( off ));
 		if ( nwritten != ( intgen_t )sizeof( off )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not write extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -657,10 +655,10 @@ dirattr_cb_extattr( dah_t dah,
 					  O_RDWR | O_CREAT,
 					  S_IRUSR | S_IWUSR );
 		if ( dtp->dt_extattrfd < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not open/create directory "
 			      "extended attributes file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -683,10 +681,10 @@ dirattr_cb_extattr( dah_t dah,
 		 */
 		seekoff = lseek64( dtp->dt_extattrfd, off, SEEK_SET );
 		if ( seekoff < 0 ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not seek to into extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -701,10 +699,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )&nextoff,
 			      sizeof( nextoff ));
 		if ( nread != ( intgen_t )sizeof( nextoff )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -718,10 +716,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )ahdrp,
 			      EXTATTRHDR_SZ );
 		if ( nread != EXTATTRHDR_SZ ) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -737,10 +735,10 @@ dirattr_cb_extattr( dah_t dah,
 			      ( void * )&ahdrp[ 1 ],
 			      recsz - EXTATTRHDR_SZ );
 		if ( nread != ( intgen_t )( recsz - EXTATTRHDR_SZ )) {
-			mlog( MLOG_NORMAL | MLOG_WARNING,
+			mlog( MLOG_NORMAL | MLOG_WARNING, _(
 			      "could not read extended attributes "
 			      "file %s: "
-			      "%s (%d)\n",
+			      "%s (%d)\n"),
 			      dtp->dt_extattrpathname,
 			      strerror( errno ),
 			      errno );
@@ -808,8 +806,8 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -820,9 +818,9 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	dirattr.d_mode = ( mode_t )fhdrp->fh_stat.bs_mode;
 	dirattr.d_uid = ( uid_t )fhdrp->fh_stat.bs_uid;
 	dirattr.d_gid = ( gid_t )fhdrp->fh_stat.bs_gid;
-	dirattr.d_atime = ( time_t )fhdrp->fh_stat.bs_atime.tv_sec;
-	dirattr.d_mtime = ( time_t )fhdrp->fh_stat.bs_mtime.tv_sec;
-	dirattr.d_ctime = ( time_t )fhdrp->fh_stat.bs_ctime.tv_sec;
+	dirattr.d_atime = ( time32_t )fhdrp->fh_stat.bs_atime.tv_sec;
+	dirattr.d_mtime = ( time32_t )fhdrp->fh_stat.bs_mtime.tv_sec;
+	dirattr.d_ctime = ( time32_t )fhdrp->fh_stat.bs_ctime.tv_sec;
 	dirattr.d_xflags = fhdrp->fh_stat.bs_xflags;
 	dirattr.d_extsize = ( u_int32_t )fhdrp->fh_stat.bs_extsize;
 	dirattr.d_dmevmask = fhdrp->fh_stat.bs_dmevmask;
@@ -835,8 +833,8 @@ dirattr_update( dah_t dah, filehdr_t *fhdrp )
 	 */
 	nwritten = write( dtp->dt_fd, ( void * )&dirattr, sizeof( dirattr ));
 	if ( ( size_t )nwritten != sizeof( dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "update of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "update of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -872,21 +870,21 @@ dirattr_get_gid( dah_t dah )
 	return dtp->dt_cached_dirattr.d_gid;
 }
 
-time_t
+time32_t
 dirattr_get_atime( dah_t dah )
 {
 	dirattr_get( dah );
 	return dtp->dt_cached_dirattr.d_atime;
 }
 
-time_t
+time32_t
 dirattr_get_mtime( dah_t dah )
 {
 	dirattr_get( dah );
 	return dtp->dt_cached_dirattr.d_mtime;
 }
 
-time_t
+time32_t
 dirattr_get_ctime( dah_t dah )
 {
 	dirattr_get( dah );
@@ -967,8 +965,8 @@ dirattr_get( dah_t dah )
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -980,8 +978,8 @@ dirattr_get( dah_t dah )
 		      ( void * )&dtp->dt_cached_dirattr,
 		      sizeof( dtp->dt_cached_dirattr ));
 	if ( ( size_t )nread != sizeof( dtp->dt_cached_dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "read of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "read of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -1045,8 +1043,8 @@ dirattr_flush( void )
 	 */
 	newoff = lseek64( dtp->dt_fd, argoff, SEEK_SET );
 	if ( newoff == ( off64_t )-1 ) {
-		mlog( MLOG_NORMAL,
-		      "lseek of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "lseek of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
@@ -1058,8 +1056,8 @@ dirattr_flush( void )
 			  ( void * )&dtp->dt_cached_dirattr,
 			  sizeof( dtp->dt_cached_dirattr ));
 	if ( ( size_t )nwritten != sizeof( dtp->dt_cached_dirattr )) {
-		mlog( MLOG_NORMAL,
-		      "flush of dirattr failed: %s\n",
+		mlog( MLOG_NORMAL, _(
+		      "flush of dirattr failed: %s\n"),
 		      strerror( errno ));
 		ASSERT( 0 );
 	}
